@@ -117,7 +117,9 @@ class BlockElement(JsonObject, metaclass=ABCMeta):
             return None
 
     @classmethod
-    def parse_all(cls, block_elements: Sequence[Union[dict, "BlockElement"]]) -> List["BlockElement"]:
+    def parse_all(
+        cls, block_elements: Sequence[Union[dict, "BlockElement", TextObject]]
+    ) -> List[Union["BlockElement", TextObject]]:
         return [cls.parse(e) for e in block_elements or []]
 
 
