@@ -36,9 +36,7 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
             with sqlite3.connect(database=self.database) as conn:
                 cur = conn.execute("select count(1) from slack_installations;")
                 row_num = cur.fetchone()[0]
-                self.logger.debug(
-                    f"{row_num} installations are stored in {self.database}"
-                )
+                self.logger.debug(f"{row_num} installations are stored in {self.database}")
         except Exception:  # skipcq: PYL-W0703
             self.create_tables()
         self.init_called = True
@@ -65,9 +63,13 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                 bot_id text not null,
                 bot_user_id text not null,
                 bot_scopes text,
+                bot_refresh_token text,  -- since v3.8
+                bot_token_expires_at datetime,  -- since v3.8
                 user_id text not null,
                 user_token text,
                 user_scopes text,
+                user_refresh_token text,  -- since v3.8
+                user_token_expires_at datetime,  -- since v3.8
                 incoming_webhook_url text,
                 incoming_webhook_channel text,
                 incoming_webhook_channel_id text,
@@ -103,6 +105,8 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                 bot_id text not null,
                 bot_user_id text not null,
                 bot_scopes text,
+                bot_refresh_token text,  -- since v3.8
+                bot_token_expires_at datetime,  -- since v3.8
                 is_enterprise_install boolean not null default 0,
                 installed_at datetime not null default current_timestamp
             );
@@ -124,52 +128,11 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
     async def async_save(self, installation: Installation):
         return self.save(installation)
 
+    async def async_save_bot(self, bot: Bot):
+        return self.save_bot(bot)
+
     def save(self, installation: Installation):
         with self.connect() as conn:
-            conn.execute(
-                """
-                insert into slack_bots (
-                    client_id,
-                    app_id,
-                    enterprise_id,
-                    enterprise_name,
-                    team_id,
-                    team_name,
-                    bot_token,
-                    bot_id,
-                    bot_user_id,
-                    bot_scopes,
-                    is_enterprise_install
-                )
-                values
-                (
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?,
-                    ?
-                );
-                """,
-                [
-                    self.client_id,
-                    installation.app_id,
-                    installation.enterprise_id or "",
-                    installation.enterprise_name,
-                    installation.team_id or "",
-                    installation.team_name,
-                    installation.bot_token,
-                    installation.bot_id,
-                    installation.bot_user_id,
-                    ",".join(installation.bot_scopes),
-                    installation.is_enterprise_install,
-                ],
-            )
             conn.execute(
                 """
                 insert into slack_installations (
@@ -184,9 +147,13 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                     bot_id,
                     bot_user_id,
                     bot_scopes,
+                    bot_refresh_token,  -- since v3.8
+                    bot_token_expires_at,  -- since v3.8
                     user_id,
                     user_token,
                     user_scopes,
+                    user_refresh_token,  -- since v3.8
+                    user_token_expires_at,  -- since v3.8
                     incoming_webhook_url,
                     incoming_webhook_channel,
                     incoming_webhook_channel_id,
@@ -215,6 +182,10 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                     ?,
                     ?,
                     ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
                     ?
                 );
                 """,
@@ -230,11 +201,13 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                     installation.bot_id,
                     installation.bot_user_id,
                     ",".join(installation.bot_scopes),
+                    installation.bot_refresh_token,
+                    installation.bot_token_expires_at,
                     installation.user_id,
                     installation.user_token,
-                    ",".join(installation.user_scopes)
-                    if installation.user_scopes
-                    else None,
+                    ",".join(installation.user_scopes) if installation.user_scopes else None,
+                    installation.user_refresh_token,
+                    installation.user_token_expires_at,
                     installation.incoming_webhook_url,
                     installation.incoming_webhook_channel,
                     installation.incoming_webhook_channel_id,
@@ -248,6 +221,62 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
             )
             conn.commit()
 
+        self.save_bot(installation.to_bot())
+
+    def save_bot(self, bot: Bot):
+        with self.connect() as conn:
+            conn.execute(
+                """
+                insert into slack_bots (
+                    client_id,
+                    app_id,
+                    enterprise_id,
+                    enterprise_name,
+                    team_id,
+                    team_name,
+                    bot_token,
+                    bot_id,
+                    bot_user_id,
+                    bot_scopes,
+                    bot_refresh_token,  -- since v3.8
+                    bot_token_expires_at,  -- since v3.8
+                    is_enterprise_install
+                )
+                values
+                (
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?,
+                    ?
+                );
+                """,
+                [
+                    self.client_id,
+                    bot.app_id,
+                    bot.enterprise_id or "",
+                    bot.enterprise_name,
+                    bot.team_id or "",
+                    bot.team_name,
+                    bot.bot_token,
+                    bot.bot_id,
+                    bot.bot_user_id,
+                    ",".join(bot.bot_scopes),
+                    bot.bot_refresh_token,
+                    bot.bot_token_expires_at,
+                    bot.is_enterprise_install,
+                ],
+            )
+            conn.commit()
+
     async def async_find_bot(
         self,
         *,
@@ -285,6 +314,8 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                         bot_id,
                         bot_user_id,
                         bot_scopes,
+                        bot_refresh_token,  -- since v3.8
+                        bot_token_expires_at,  -- since v3.8
                         is_enterprise_install,
                         installed_at
                     from
@@ -302,9 +333,7 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                 )
                 row = cur.fetchone()
                 result = "found" if row and len(row) > 0 else "not found"
-                self.logger.debug(
-                    f"find_bot's query result: {result} (database: {self.database})"
-                )
+                self.logger.debug(f"find_bot's query result: {result} (database: {self.database})")
                 if row and len(row) > 0:
                     bot = Bot(
                         app_id=row[0],
@@ -316,15 +345,20 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                         bot_id=row[6],
                         bot_user_id=row[7],
                         bot_scopes=row[8],
-                        is_enterprise_install=row[9],
-                        installed_at=row[10],
+                        bot_refresh_token=row[9],
+                        bot_token_expires_at=row[10],
+                        is_enterprise_install=row[11],
+                        installed_at=row[12],
                     )
                     return bot
                 return None
 
         except Exception as e:  # skipcq: PYL-W0703
             message = f"Failed to find bot installation data for enterprise: {enterprise_id}, team: {team_id}: {e}"
-            self.logger.warning(message)
+            if self.logger.level <= logging.DEBUG:
+                self.logger.exception(message)
+            else:
+                self.logger.warning(message)
             return None
 
     async def async_find_installation(
@@ -367,9 +401,13 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                     bot_id,
                     bot_user_id,
                     bot_scopes,
+                    bot_refresh_token,  -- since v3.8
+                    bot_token_expires_at,  -- since v3.8
                     user_id,
                     user_token,
                     user_scopes,
+                    user_refresh_token,  -- since v3.8
+                    user_token_expires_at,  -- since v3.8
                     incoming_webhook_url,
                     incoming_webhook_channel,
                     incoming_webhook_channel_id,
@@ -423,9 +461,7 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                     return None
 
                 result = "found" if row and len(row) > 0 else "not found"
-                self.logger.debug(
-                    f"find_installation's query result: {result} (database: {self.database})"
-                )
+                self.logger.debug(f"find_installation's query result: {result} (database: {self.database})")
                 if row and len(row) > 0:
                     installation = Installation(
                         app_id=row[0],
@@ -438,28 +474,69 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                         bot_id=row[7],
                         bot_user_id=row[8],
                         bot_scopes=row[9],
-                        user_id=row[10],
-                        user_token=row[11],
-                        user_scopes=row[12],
-                        incoming_webhook_url=row[13],
-                        incoming_webhook_channel=row[14],
-                        incoming_webhook_channel_id=row[15],
-                        incoming_webhook_configuration_url=row[16],
-                        is_enterprise_install=row[17],
-                        token_type=row[18],
-                        installed_at=row[19],
+                        bot_refresh_token=row[10],
+                        bot_token_expires_at=row[11],
+                        user_id=row[12],
+                        user_token=row[13],
+                        user_scopes=row[14],
+                        user_refresh_token=row[15],
+                        user_token_expires_at=row[16],
+                        incoming_webhook_url=row[17],
+                        incoming_webhook_channel=row[18],
+                        incoming_webhook_channel_id=row[19],
+                        incoming_webhook_configuration_url=row[20],
+                        is_enterprise_install=row[21],
+                        token_type=row[22],
+                        installed_at=row[23],
                     )
+
+                    if user_id is not None:
+                        # Retrieve the latest bot token, just in case
+                        # See also: https://github.com/slackapi/bolt-python/issues/664
+                        cur = conn.execute(
+                            """
+                            select
+                                bot_token,
+                                bot_id,
+                                bot_user_id,
+                                bot_scopes,
+                                bot_refresh_token,
+                                bot_token_expires_at
+                            from
+                                slack_installations
+                            where
+                                client_id = ?
+                                and
+                                enterprise_id = ?
+                                and
+                                team_id = ?
+                                and
+                                bot_token is not null
+                            order by installed_at desc
+                            limit 1
+                            """,
+                            [self.client_id, enterprise_id or "", team_id],
+                        )
+                        row = cur.fetchone()
+                        installation.bot_token = row[0]
+                        installation.bot_id = row[1]
+                        installation.bot_user_id = row[2]
+                        installation.bot_scopes = row[3]
+                        installation.bot_refresh_token = row[4]
+                        installation.bot_token_expires_at = row[5]
+
                     return installation
                 return None
 
         except Exception as e:  # skipcq: PYL-W0703
             message = f"Failed to find an installation data for enterprise: {enterprise_id}, team: {team_id}: {e}"
-            self.logger.warning(message)
+            if self.logger.level <= logging.DEBUG:
+                self.logger.exception(message)
+            else:
+                self.logger.warning(message)
             return None
 
-    def delete_bot(
-        self, *, enterprise_id: Optional[str], team_id: Optional[str]
-    ) -> None:
+    def delete_bot(self, *, enterprise_id: Optional[str], team_id: Optional[str]) -> None:
         try:
             with self.connect() as conn:
                 conn.execute(
@@ -479,7 +556,10 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                 conn.commit()
         except Exception as e:  # skipcq: PYL-W0703
             message = f"Failed to delete bot installation data for enterprise: {enterprise_id}, team: {team_id}: {e}"
-            self.logger.warning(message)
+            if self.logger.level <= logging.DEBUG:
+                self.logger.exception(message)
+            else:
+                self.logger.warning(message)
 
     def delete_installation(
         self,
@@ -525,4 +605,7 @@ class SQLite3InstallationStore(InstallationStore, AsyncInstallationStore):
                 conn.commit()
         except Exception as e:  # skipcq: PYL-W0703
             message = f"Failed to delete installation data for enterprise: {enterprise_id}, team: {team_id}: {e}"
-            self.logger.warning(message)
+            if self.logger.level <= logging.DEBUG:
+                self.logger.exception(message)
+            else:
+                self.logger.warning(message)
